package org.eclipse.jdt.internal.compiler.lookup;

import java.util.Set;
import java.util.HashSet;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

abstract class ConstraintFormula extends ReductionResult
{
    static final List<InferenceVariable> EMPTY_VARIABLE_LIST;
    static final ConstraintFormula[] NO_CONSTRAINTS;
    static final char LEFT_ANGLE_BRACKET = '⟨';
    static final char RIGHT_ANGLE_BRACKET = '⟩';
    
    static {
        EMPTY_VARIABLE_LIST = Collections.emptyList();
        NO_CONSTRAINTS = new ConstraintTypeFormula[0];
    }
    
    ConstraintFormula() {
    }
    
    public abstract Object reduce(InferenceContext18 p0) throws InferenceFailureException;
    
    Collection<InferenceVariable> inputVariables(InferenceContext18 context) {
        return ConstraintFormula.EMPTY_VARIABLE_LIST;
    }
    
    Collection<InferenceVariable> outputVariables(InferenceContext18 context) {
        Set<InferenceVariable> variables = new HashSet();
        this.right.collectInferenceVariables(variables);
        if (!variables.isEmpty()) {
            variables.removeAll(this.inputVariables(context));
            return variables;
        }
        return variables;
    }
    
    public boolean applySubstitution(BoundSet solutionSet, InferenceVariable[] variables) {
        int i = 0;
        while (i < variables.length) {
            InferenceVariable variable = variables[i];
            TypeBinding instantiation = solutionSet.getInstantiation(variables[i], null);
            if (instantiation == null) {
                return false;
            }
            this.right = this.right.substituteInferenceVariable(variable, instantiation);
            ++i;
        }
        return true;
    }
    
    protected void appendTypeName(StringBuffer buf, TypeBinding type) {
        if ((type instanceof CaptureBinding18)) {
            buf.append(type.toString());
        }
        else {
            buf.append(type.readableName());
        }
    }
}
